# -*- coding: iso-8859-1 -*-
#-----------------------------------------------------------------------------
#
# Copyright  2012 Stephen Baumgartner <steve@slbaumgartner.com>
#
# Permission to use, copy, modify, and distribute this software for
# any purpose and without fee is hereby granted, provided the above
# copyright notice appears in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#-----------------------------------------------------------------------------
#
# Name:           Dimension Utilities
# Description:    Adds two tools to the Plugins menu:
#                 "Select all Dimensions", and "Move Dimensions to Layer".
#                 These tools simplify tasks commonly needed in the following
#                 workflow:  Layer0 is the only layer ever chosen as "active", so
#                 all Drawingelements are initially created on Layer0.  As the model
#                 increases in complexity, additional layers may be added and Groups
#                 and Components moved to them to selectively control visibility.
#                 When the model is complete, various scenes/pages are created to
#                 emphasize desired aspects of the model.  Among these may be scenes
#                 that provide dimensions.  Like the other elements, the dimensions
#                 are all created on Layer0.  However, dimensions that are useful on
#                 one scene may be clutter on another, so they are moved to layers that
#                 are visible only on the appropriate scene(s).  Done manually, this
#                 operation is tedious and prone to error.  This Ruby automates key steps.
# Usage:          "Select all Dimensions" activates and clears the Select tool and
#                 then adds all currently visible (in the active scene) linear
#                 dimensions, Text objects, and Groups with names matching
#                 /Angular Dimension/.  This prepares the scene for bulk manipulations
#                 of dimensions, such as changing font, size, assignment of layer, etc.
#                 (the feasible manipulations depend on what kinds of dimensions are present).
#                 "Move Dimensions to Layer" starts by prompting the user to select an
#                 existing layer or to enter a name for a new layer.  If a new layer
#                 is specified, it is created and hidden on all scenes except the
#                 currently active scene.  All currently visible linear dimensions,
#                 Text objects, and Groups with names matching /Angular Dimension/
#                 are then moved to the selected layer.
# Note:           Groups named "Angular Dimension" are created by my other tool,
#                 draw_angular_dimension.rb
#-----------------------------------------------------------------------------
require 'sketchup'

module DimSelector
  def select_all_dims
    model = Sketchup.active_model
    
    # this causes Sketchup to switch immediately to the Select tool
    # without this a different active tool could interfere with our action
    # (the Dimension tool is a prime offender)
    model.select_tool nil
    
    ents = model.entities
    sel = model.selection
    sel.clear
    
    ents.each { |ent|
      # Check first for Drawingelement since not all Entities will respond
      # to .visible? or .layer.  Shortcut logic && prevents testing other Entities.
      # The hidden flag for an entity is independent of the hidden flag for
      # its layer, so we test both to make sure we get only visible entities
      if(ent.kind_of?(Sketchup::Drawingelement) && ent.visible? && ent.layer.visible?)
        # typename is less efficient than instance_of?, but linear dimensions do not
        # have a class of their own
        if(ent.typename == "DimensionLinear")
          sel.add(ent)
          elsif(ent.instance_of?(Sketchup::Text))
          sel.add(ent)
          elsif(ent.instance_of?(Sketchup::Group) && ent.name =~ /Angular Dimension/)
          sel.add(ent)
        end
      end
    }
  end
end

module DimsToLayer
  def move_dims_to_layer
    ents = Sketchup.active_model.entities
    to_layer = get_layer
    if(to_layer != nil)
      Sketchup.active_model.start_operation "Dimensions to Layer"
      ents.each { |ent|
        # Check first for Drawingelement since not all Entities will respond
        # to visible? or layer
        # The hidden flag for an entity is independent of the hidden flag for
        # its layer, so we test both to make sure we get only visible entities
        if(ent.kind_of?(Sketchup::Drawingelement) && ent.visible? && ent.layer.visible?)
          # typename is less efficient than instance_of?, but linear dimensions do not
          # have a class of their own
          if(ent.typename == "DimensionLinear")
            ent.layer=to_layer
            elsif(ent.instance_of?(Sketchup::Text))
            ent.layer=to_layer
            elsif(ent.instance_of?(Sketchup::Group) && ent.name =~ /Angular Dimension/)
            ent.layer=to_layer
          end
        end
      }
      Sketchup.active_model.commit_operation
    end
  end
  
  def get_layer
    model = Sketchup.active_model
    # build the list of available layers for the input box
    layers = model.layers
    layer_list = []
    layers.each {|layer|
      layer_list << layer.name
    }
    layer_list.sort! {|a,b| a.downcase <=> b.downcase}
    list_str = "New Layer|" + layer_list.join("|")
    list = [list_str]
    prompts = ["Choose a layer "]
    defaults = ["New Layer"]
    input_arr = UI.inputbox prompts, defaults, list, "Target Layer for Dimensions"
    if(input_arr)
      if(input_arr[0] == "New Layer")
        ret = get_new_layer
        else
        # user chose an existing layer - return it
        ret = layers[input_arr[0]]
      end
      else
      # user cancelled selection list
      ret = nil
    end
    return ret
  end
  
  def get_new_layer
    model = Sketchup.active_model
    layers = model.layers
    new_layer_name_arr = UI.inputbox ["Name for New Layer"]
    if(new_layer_name_arr)
      # user entered a new layer name
      new_layer_name = new_layer_name_arr[0].strip
      if(layers[new_layer_name])
        UI.messagebox(new_layer_name + "\nis already in use - quitting")
        ret = nil
        else
        # create a new layer and hide it on all except the current page
        model.start_operation "New Layer for Dimensions"
        new_layer = layers.add(new_layer_name)
        new_layer.page_behavior = LAYER_IS_HIDDEN_ON_NEW_PAGES
        pages = model.pages
        curr_page = pages.selected_page
        pages.each { |p| p.set_visibility( new_layer, false ) if (p!=curr_page)}
        model.commit_operation
        ret = new_layer
      end
      else
      # user cancelled new name input
      ret = nil
    end
    return ret
  end
end

if( not file_loaded?("dimension_utilities.rb") )
  include DimSelector
  selcmd = UI::Command.new("SelectDims") { DimSelector::select_all_dims }
  selcmd.menu_text = "Select All Dimensions"
  UI.menu("Plugins").add_item selcmd
  
  include DimsToLayer
  movcmd = UI::Command.new("DimsToLayer") { DimsToLayer::move_dims_to_layer }
  movcmd.menu_text = "Move Dimensions to Layer"
  UI.menu("Plugins").add_item movcmd
end
#-----------------------------------------------------------------------------
file_loaded("dimension_utilities.rb")